/* CALLBACK
 *
 * This file contains a few functions that nearly every daemon needs. Often
 * you will need to alter the functions contained herein. You should see this
 * file as a code template. It works as is, modify it to suit your needs.
 *
 * This file is part of the Daemon server product.
 * Copyright (C) ITB CompuPhase
 * Permission is granted to use or modify this file. There is no warranty of
 * any kind.
 */

#include <string.h>
#include "daemon.h"


/*
 * The call back routine for the Daemon server is not really an interrupt
 * routine, but for convenience, the Daemon server simulates and interrupt. See
 * "Undocumented DOS" chapter 6 (in the first edition) or chapter 9 (second
 * edition) for this structure.
 */
struct INTERRUPT_REGS {
  #if defined(__TURBOC__)
    unsigned short bp,di,si,ds,es,dx,cx,bx,ax;
  #else
    unsigned short es,ds;
    unsigned short di,si,bp,sp,bx,dx,cx,ax;     /* PUSHA */
  #endif
  unsigned short ip,cs,flags;
};


/*
 * The Daemon server uses a register based calling convention. For convenience,
 * a glue routine in DLIB.OBJ has pushed the registers to the stack. It uses
 * the same technique as Microsoft's and Borland's C compilers do for interrupt
 * routines.
 *
 * When this function is called, you must be carefull with what you do. In
 * order to access static (or global) variables, must must switch to your data
 * segment. But if you do that, the data segment and the heap/stack segment
 * will not be the same. As said earlier, the glue function in DLIB simulates
 * an interrupt, so consult your compiler's documentation on how to handle
 * interrupt routines.
 */
void _interrupt _far d_callback(struct INTERRUPT_REGS r)
{
  extern D_TABLE d_table;               /* must be declared in the daemon */
  char _far *str;
  int i;

  switch (r.ax & 0x00ff) {              /* check low byte only */
    case D_SYSREQ_INIT:
      /* If your daemon requires other daemons, load them here */
      break;

    case D_SYSREQ_EXIT:
      /* If you loaded daemons in D_SYSREQ_INIT, unload them here */
      break;

    case D_SYSREQ_ADDR:
      str = MK_FP(r.es, r.di);          /* get pointer to function name */
      for (i = 0; d_table[i].name != 0; i++) {
        if (_fstrcmp(d_table[i].name, str) == 0) {
          r.dx = FP_SEG(d_table[i].func);
          r.ax = FP_OFF(d_table[i].func);
          return;                       /* function address in DX:AX */
        } /* if */
      } /* for */
      r.dx = r.ax = 0;                  /* return NULL pointer */
      break;

    case D_SYSREQ_VER:
      r.ax = 0x100;                     /* version 1.0 */
      break;

  } /* switch */
}


/*
 * Microsoft C can give run time errors on startup, because the default
 * command line processing tries to expand the memory block it is in. This
 * is not always possible. Therefore, we create empty routines that replace
 * the library routines.
 */
#if defined(_MSC_VER)
void _setargv() {}
void _setenvp() {}
#endif

