/* UUDECODE
 *
 * This example program shows how to use load a daemon and use its functions.
 *
 * This application is a UUDECODE utility, based on the simple routines in
 * UULIB.EXE. (This is definitely not an industry strength UUDECODE utlity).
 * See UULIB.C for information on encoding and decoding.
 *
 * To build the program, compile and link with DLIB.OBJ.
 *
 *
 * This file is part of the Daemon server product.
 * Copyright (C) ITB CompuPhase
 * Permission is granted to use or modify this file. There is no warranty of
 * any kind.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include "daemon.h"
#include "uulib.h"

/*
 * If you are using Borland C++, the small or medium memory model and you need
 * a heap, you must set it explicitly. Otherwise the Daemon server assumes a
 * 64 kBytes heap. It is better to set the stack length as well.
 *
 * You may set the stack in the compact, large and huge memory models too, but
 * these memory models use a different heap strategy and _heaplen is ignored.
 *
 * None of this applies to Microsoft C.
 */
#if defined (__TURBOC__)
  extern unsigned _stklen = 2048;       /* sufficient for this program */
  #if defined(__SMALL__) || defined(__MEDIUM__)
    extern unsigned _heaplen = 1024;    /* very small heap */
  #endif
#endif


/*
 * Function prototypes for internal functions.
 */
void decode(char *source);
void encode(char *source, char *target);


int main(int argc, char *argv[])
{
  extern D_TABLE uulib_table;           /* from UULIB&.OBJ */
  short uulib;
  char *ext, filename[128], dest_file[128];

  if (argc == 1) {
    printf("Usage: uuencode <filename> [<filename> ...]\n\n"
           "If the extension of a file is .UUE, it will be decoded; otherwise\n"
           "the file is encoded\n");
    return 1;
  } /* if */

  if (!d_init()) {
    printf("Daemon server not present\n");
    return 1;
  } /* if */

  /*
   * Load the daemon (from the same file as this program). Load the daemon in
   * an UMB (if available) and as high as possible. Store the daemon number.
   * Then link the functions.
   */
  strcpy(filename, argv[0]);
  strcat(filename, "/uulib.exe");
  if ((uulib = d_load(filename, D_MEM_UMB | D_MEM_LASTFIT)) > 0xff) {
    printf("Error %d in loading \"%s\" daemon\n", uulib >> 8, filename);
    return 1;
  } /* if */
  if (!d_linkall(uulib, uulib_table)) {
    printf("Link table invalid\n");
    return 1;
  } /* if */

  while (--argc > 0) {
    strcpy(filename, *++argv);
    ext = strrchr(filename, '.');
    if (ext != NULL && strchr(ext,'\\') == NULL && stricmp(ext,".UUE") == 0) {
      /*
       * Decode file
       */
      decode(filename);
    } else {
      /*
       * Encode file
       */
      strcpy(dest_file, filename);
      if (ext == NULL || strchr(ext,'\\') != NULL)
        strcat(dest_file, ".UUE");              /* file had no extension */
      else
        strcpy(strrchr(dest_file,'.'), ".UUE"); /* replace existin extension */
      encode(filename, dest_file);
    } /* if */
  } /* while */

  /*
   * Unload the daemon, and quit
   */
  d_unload(uulib);
  return 0;
}


void decode(char *source)
{
  FILE *in, *out;
  int len;
  char dest[128];
  char txt_buf[80];
  unsigned char bin_buf[80];

  /* open input file */
  if ((in = fopen(source, "rt")) == NULL) {
    perror(source);
    exit(1);
  } /* if */

  /* search for header line */
  for ( ;; ) {
    if (fgets(txt_buf, sizeof txt_buf, in) == NULL) {
      printf("No \"begin\" line\n");
      exit(1);
    } /* if */
    if (strncmp(txt_buf, "begin ", 6) == 0)
      break;
  } /* for */
  sscanf(txt_buf, "begin %*o %s", dest);
  printf("Processing file \"%s\"\n", dest);

  /* create output file (binary) */
  if ((out = fopen(dest, "wb")) == NULL) {
    perror(dest);
    exit(1);
  } /* if */

  /* decode the file */
  for ( ;; ) {
    /* for each input line */
    if (fgets(txt_buf, sizeof txt_buf, in) == NULL)
      break;
    len = uudecode(bin_buf, txt_buf);
    if (len == 0)
      break;
    fwrite(bin_buf, 1, len, out);
  } /* for */

  if (fgets(txt_buf, sizeof txt_buf, in) == NULL || strcmp(txt_buf+1, "end") == 0)
    printf("Short file (no \"end\" line\n");

  fclose(in);
  fclose(out);
}


void encode(char *source, char *destination)
{
  FILE *in, *out;
  size_t len;
  char txt_buf[80];
  unsigned char bin_buf[45];

  /* open input file */
  if ((in = fopen(source, "rb")) == NULL) {
    perror(source);
    exit(1);
  } /* if */

  /* create output file (text) */
  if ((out = fopen(destination, "wt")) == NULL) {
    perror(destination);
    exit(1);
  } /* if */

  /* create header line */
  fprintf(out, "begin 644 %s\n", source);
  printf("Processing file \"%s\"\n", source);

  /* encode the file */
  do {
    len = fread(bin_buf, 1, 45, in);    /* read 45 bytes (at most) */
    uuencode(txt_buf, bin_buf, len);
    fprintf(out, "%s\n", txt_buf);
  } while (len == 45);

  /* end the file */
  fprintf(out, "`\n"                    /* back quote ends the stream */
               "end\n");                /* keyword ends the file */
  fclose(in);
  fclose(out);
}

